function [Q,R]=myqr(A,flag)   
%QR	Computes QR decomposition of A. 
% 	It uses the Gram-Schmidt algorithm. 
%	The inner product is given by <x,y>=x^ty. 
% 	The command is [Q,R]=myqr(A,flag). 
%   If flag=0, the "economy size" answer is provided

%   based upon GRAM.M, Copyright 1993 Terry Lawson
%	Terry Lawson, Math Department, Tulane University, 11/93

%   Modified by CLV, 20061112
%       Nows deals correctly with rank deficient matrix A
%           Tests performed: 
%               A=builtin('eye',5);A(:,6:8)=A(:,1:3);%m<n; first columns linearly independent
%               [Q,R]=qr(A+mp(0),0);[QQ,RR]=qr(A,0); double(norm(Q*R-A,'inf'))
% 		
%               A(3,3)=0;                           %m<n; not all first columns linearly independent
%               [Q,R]=qr(A+mp(0),0);[QQ,RR]=qr(A,0); double(norm(Q*R-A,'inf'))
% 		
%               A(3,3)=1;A(6,:)=builtin('ones',1,8);%m<n, rank deficient
%               [Q,R]=qr(A+mp(0),0);[QQ,RR]=qr(A,0); double(norm(Q*R-A,'inf'))
% 		
%               A=A';                               %m>n, activate "economy size" option
%               [Q,R]=qr(A+mp(0),0);[QQ,RR]=qr(A,0); double(norm(Q*R-A,'inf'))
%
%   Modified by CLV, 20041229
%       "flag" is added as a parameter, to specify the "economy size" decomposition or not

if (nargin < 2), flag = 1;end %request full size decomposition
[m,n]=size(A);
zero=A(1)*0; %a zero of the same data type as matrix A
switch class(A)
    case 'double'
        EPS=eps;
    otherwise
        EPS=eps(zero+1);
end
tol = max([m,n])*EPS*norm(A,'inf'); 
v=A;w=A;
R=repmat(zero,m,n);Q=repmat(zero,m,m);t=R;
k=1;
w(:,1)=v(:,1);
R(1,1)=sqrt(w(:,1)'*w(:,1));
Q(:,1)=w(:,1)/R(1,1);nn(1)=R(1,1);
for j= 2:min(m,n)
    wtemp = v(:,j);%The given column
    for i= 1:k
        t(i,j)=v(:,j)'*w(:,i)/(w(:,i)'*w(:,i));%the projection of v(:,j) onto w(:,i)
        wtemp = wtemp-t(i,j)*w(:,i); %now wtemp is orthogonal to all w(:,1:i) 
        R(i,j)=t(i,j)*nn(i);
    end
    k=k+1;
    
    if sqrt(wtemp'*wtemp) > tol
        w(:,k)=wtemp;
        nn(k)=sqrt(wtemp'*wtemp);
        R(k,j)=nn(k);%Unlike matlab, this is always positive!
        Q(:,k)=w(:,k)/nn(k);
    else
        %The j-th column of A belongs to the subspace defined by the 1:(j-1) ones
        %Create at random one column for Q, orthogonal to the mentioned subspace, and
        %calculate the entries of R accordingly
        [Q,nn,w,k]=fill_with_something(Q,nn,w,m,n,k,tol,zero);%last column of Q is orthogonal to the earlier ones
        R(:,k)=Q'*A(:,k);
        nn(k)=1;
        w(:,k)=Q(:,k);
    end
    %Verification: This should be zero
    %double(norm(Q(:,1:k)*R(1:k,1:k)-A(:,1:k),'inf'))
end

if n>m
    R(:,(k+1):n)=Q'*A(:,(k+1):n);
end
%exit if flag is explicitly set to zero
if (flag==0) & (m>n)
    %Return only first n columns
    Q=Q(:,1:k);
    R=R(1:k,:);
    return
end

%if m>n, we should invent something for the rest of the columns
k=n+1;
while k<=m
    [Q,nn,w,k]=fill_with_something(Q,nn,w,m,n,k,tol,zero);
    k=k+1;
end
return



function [Q,nn,w,k]=fill_with_something(Q,nn,w,m,n,k,tol,zero);
rkk=0;iter=0;maxiter=20;
while (rkk<tol & iter<maxiter)
    iter=iter+1;
    dummy=n*rand(m,1)+zero;
    for i=1:(k-1)
        rik=Q(:,i)'*dummy;
        dummy=dummy-rik*Q(:,i);
    end
    qk=dummy;
    rkk=sqrt(qk'*qk);
end
if (iter==maxiter)
    disp(['Problems in myqr:' num2str(double(rkk))])
end
Q(:,k)=qk/rkk;
nn(k)=rkk;
w(:,k)=qk;
%k=k+1;
